unit spcx;
interface
uses
  Crt, Dos;

const
   MAX_WIDTH = 4000;    { arbitrary - maximum width (in bytes) of a PCX image }
   COMPRESS_NUM = $C0;  { this is the upper two bits that indicate a count }
   MAX_BLOCK = 4096;
   RED = 0;
   GREEN = 1;
   BLUE = 2;
   MCGA = $13;
   tiling:boolean=false;

type
    memopicture=record
        gen:word;
        uz:byte;
        resim:pointer;
   end;

   str80=string[80];

procedure swpcx (x,y:integer;name: str80;trans:byte);
procedure loadtomemory(name:str80;num:byte);
procedure loadpal(name:str80);
procedure swpcxfrommemo (x,y:integer;num:byte;trans:byte);
procedure mcgamod;
procedure fadeout(d:integer);
procedure fadein(d:integer);
procedure cls(x,y,cx,cy:word);
procedure release(numara:byte);

implementation
type
 pcx_header = record
        Manufacturer: byte;     { Always 10 for PCX file }

        Version: byte;          { 2 - old PCX - no palette (not used anymore),
                                  3 - no palette,
                                  4 - Microsoft Windows - no palette (only in
                                      old files, new Windows version uses 3),
                                  5 - with palette }

        Encoding: byte;         { 1 is PCX, it is possible that we may add
                                  additional encoding methods in the future }

        Bits_per_pixel: byte;   { Number of bits to represent a pixel
                                  (per plane) - 1, 2, 4, or 8 }

        Xmin: integer;          { Image window dimensions (inclusive) }
        Ymin: integer;          { Xmin, Ymin are usually zero (not always) }
        Xmax: integer;
        Ymax: integer;

        Hdpi: integer;          { Resolution of image (dots per inch) }
        Vdpi: integer;          { Set to scanner resolution - 300 is default }

        ColorMap: array [0..15, RED..BLUE] of byte;
                                { RGB palette data (16 colors or less)
                                  256 color palette is appended to end of file }

        Reserved: byte;         { (used to contain video mode)
                                  now it is ignored - just set to zero }

        Nplanes: byte;          { Number of planes }

        Bytes_per_line_per_plane: integer;   { Number of bytes to allocate
                                               for a scanline plane.
                                               MUST be an an EVEN number!
                                               Do NOT calculate from Xmax-Xmin! }

        PaletteInfo: integer;   { 1 = black & white or color image,
                                  2 = grayscale image - ignored in PB4, PB4+
                                  palette must also be set to shades of gray! }

        HscreenSize: integer;   { added for PC Paintbrush IV Plus ver 1.0,  }
        VscreenSize: integer;   { PC Paintbrush IV ver 1.02 (and later)     }
                                { I know it is tempting to use these fields
                                  to determine what video mode should be used
                                  to display the image - but it is NOT
                                  recommended since the fields will probably
                                  just contain garbage. It is better to have
                                  the user install for the graphics mode he
                                  wants to use... }

        Filler: array [74..127] of byte;     { Just set to zeros }

        end;

   file_buffer = array [0..127] of byte;
   block_array = array [0..MAX_BLOCK] of byte;
   ega_array = array [0..16] of byte;
   line_array = array [0..MAX_WIDTH] of byte;
   pal_array = array [0..255, RED..BLUE] of byte;
var
   potr:array [1..10] of memopicture;
   l,t:word;
   k, kmax,gen,uz: integer;
   Name: str80;                        { Name of PCX file to load }
   ImageName: str80;                   { Name of PCX file - used by ReadError }
   BlockFile: file;                    { file for reading block data }
   BlockData: block_array;

   Header: pcx_header;                 { PCX file header }
   Palette256: pal_array;              { place to put 256 color palette }
   PaletteEGA: ega_array;              { place to put 17 EGA palette values }
   PCXline: line_array;                { place to put uncompressed data }

   Ymax: integer;                      { maximum Y value on screen }
   NextByte: integer;                  { index into file buffer in ReadByte }
   Index: integer;                     { PCXline index - where to put Data }
   Data: byte;                         { PCX compressed data byte }

   PictureMode: integer;               { Graphics mode number }
   Reg: Registers;
   bytes_per_line :word;

procedure read256palette;forward;

procedure Error (s: str80 );
var c: char;
    i: integer;

begin
TextMode (C80);
writeln ('HATA');
writeln (s);
halt;
end;

procedure ReadError (msg: integer);
begin
if IOresult <> 0 then

   case msg of
   1: Error ('Dosya bulunamad - ' + ImageName);
   2: Error ('Dosya kapatlrken hata olutu - ' + ImageName + ' - disk dolu' );
   3: Error ('Dosya okunamyor! - ' + ImageName);

   else
      Error ('Dosya I/O  hatas- ' + ImageName);
   end;
end;

{*************************** read header ***********************************}

procedure ReadHeader;
label WrongFormat;
begin
{$I-}
BlockRead (BlockFile, Header, 128);         { read 128 byte PCX header }
ReadError (3);
if (Header.Manufacturer <> 10) or (Header.Encoding <> 1) then
   begin
   close (BlockFile);
   Error ('This is not a valid PCX image file.');
   end;
if (Header.Nplanes = 4) and (Header.Bits_per_pixel = 1) then
   begin
   if (Header.Ymax - Header.Ymin) <= 349 then
      begin

      Ymax := 349;
      end
   else
      begin

      Ymax := 479;
      end;
   end
else if (Header.Nplanes = 1) then
   begin
   Ymax := 199;

   if (Header.Bits_per_pixel = 1) then

   else if (Header.Bits_per_pixel = 2) then

   else if (Header.Bits_per_pixel = 8) then
      begin
      PictureMode := MCGA;
      if Header.Version = 5 then
         Read256palette;
      end
   else
      goto WrongFormat;
   end
else
   begin
WrongFormat:
   close (BlockFile);
   Error ('PCX file is in wrong format - It must be a CGA, EGA, VGA, or MCGA image');
   end;

Index := 0;
NextByte := MAX_BLOCK;          { indicates no data read in yet... }
l := Header.XMax - Header.Xmin;
t:=header.ymax-header.ymin;
bytes_per_line := Header.Bytes_per_line_per_plane * Header.Nplanes;

end;  { ReadHeader }


procedure EntireVGApalette;
var
   i: integer;
begin
for i := 0 to 255 do
   begin
   Palette256 [i, RED]   := Palette256 [i, RED]   shr 2;
   Palette256 [i, GREEN] := Palette256 [i, GREEN] shr 2;
   Palette256 [i, BLUE]  := Palette256 [i, BLUE]  shr 2;
   end;
Reg.ah := $10;                      { Set DAC Call }
Reg.al := $12;                      { set a block of DAC registers}
Reg.bx := 0;                       { first DAC register numbe     }
Reg.cx := 255;                      { number of registers to update }
Reg.dx := ofs (Palette256);          { offset of block }
Reg.es := seg (Palette256);          { segment of block }
intr ($10, Reg);                     { call interrupt }

end;

procedure ShowMCGA (gx,gY: integer);
var
   i:integer;
   MCGAscreen: array [0..64000] of byte absolute $A000:0000;
begin
l:=gen-1;
if gx+l>320 then l:=320-gx;
if tiling then l:=320;
if gx>=0 then
begin
move(pcxline[0],mcgascreen[gy*320+gx],l);
end
  else
  begin
  l:=gen+gx-1;
  move(pcxline[abs(gx)],mcgascreen[gy*320],l);
  end;
end;   { ShowMCGA }

procedure transput (gx,gy: integer);
var
   i:integer;
   MCGAscreen: array [0..64000] of byte absolute $A000:0000;
begin
l:=gen-1;
if l > 320 then
   l := 320-gx;
if gx+l>320 then l:=320-gx;
if tiling then l:=320;
if gx>=0 then begin
for i:=0 to l do
if pcxline[i]<>0 then MCGAScreen [gy * 320+gx+i]:=pcxline[i];
end
else begin
for i:=abs(gx) to gen-1 do
  if pcxline[i]<>0 then MCGAScreen[gy * 320+i-abs(gx)]:=pcxline[i];
end;
end;   { ShowMCGA }

procedure Read256palette;
var
   i: integer;
   b: byte;

begin
seek (BlockFile, FileSize (BlockFile) - 769);
BlockRead (BlockFile, b, 1);           { read indicator byte }
ReadError (3);
if b <> 12 then  {no palette here... }
   exit;
BlockRead (BlockFile, Palette256, 3*256);
ReadError (3);
seek (BlockFile, 128);                 { go back to start of PCX data }
end;  { Read256palette }

{*************************** read byte *************************************}

procedure ReadByte;
var
   NumBlocksRead: integer;

begin
if NextByte = MAX_BLOCK then
   begin
   BlockRead (BlockFile, BlockData, MAX_BLOCK, NumBlocksRead);
   NextByte := 0;
   end;
data := BlockData [NextByte];
inc (NextByte);                         { NextByte++; }
end;  { ReadByte }

{*************************** read pcx line *********************************}

procedure Read_PCX_Line;
var
   count: integer;
begin
{$I-}
if Index <> 0 then
   FillChar (PCXline [0],Index,data);    { fills a contiguous block of data }
while (Index < bytes_per_line) do          { read 1 line of data (all planes) }
   begin
   ReadByte;
if (data and $c0) = compress_num then
      begin
      count := data and $3F;
      ReadByte;
      FillChar (PCXline [Index], count, data);  { fills a contiguous block }
      inc (Index, count);                       { Index += count; }
      end
   else
      begin
      PCXline [Index] := data;
      inc (Index);                              { Index++; }
      end;
   end;

ReadError (3);
Index := Index - bytes_per_line;
{$I+}
end;

procedure loadtomemory;
var
   i:integer;
begin
ImageName := name;
assign (BlockFile, name);
{$i-} reset (BlockFile, 1); {$i+}
if ioresult<>0 then begin
                          textmode(3);
                          writeln('Dosya bulunamad : ',name);
                          halt;
                   end;

ReadError (1);
ReadHeader;
if header.xmax > 310 then header.xmax:=320;
potr[num].gen:=header.xmax-header.xmin;
potr[num].uz := Header.Ymin+Ymax;
getmem(potr[num].resim,potr[num].uz*potr[num].gen);
if Header.Ymax<potr[num].uz then
   potr[num].uz := Header.ymax;
     for i := 0 to potr[num].uz do
      begin
      Read_PCX_Line;
      move(pcxline[0],mem[seg(potr[num].resim^):ofs(potr[num].resim^)+potr[num].gen * i],potr[num].gen);
      end;
close (BlockFile);
ReadError (2);
end;

procedure swpcxfrommemo;
var
  screen:array [0..64000] of byte absolute $A000:0000;
begin
uz:=potr[num].uz;
gen:=potr[num].gen;
kmax:=potr[num].uz;
if y+kmax>200 then kmax:=200-y;
  for k := 0 to kmax do
      begin
       move(mem[seg(potr[num].resim^):ofs(potr[num].resim^)+k * (potr[num].gen)],pcxline[0],potr[num].gen);
       if trans=0 then showmcga(x,y+k) else transput(x,y+k);
   end;
end;

procedure swpcx;
begin
{$I-}
ImageName := name;                     { used by ReadError }
assign (BlockFile, name);
reset (BlockFile, 1);                  { use 1 byte blocks }
ReadError (1);
ReadHeader;
gen:=header.xmax-header.xmin;
uz:=header.ymax-header.ymin;
if (header.version=5)  and (trans<>1) then entirevgapalette;
kmax := Header.Ymin + Ymax;
if Header.Ymax < kmax then  { Ekrann alabileceinden fazla gsterme }
   kmax := Header.ymax;
   if y+kmax>200 then kmax:=200-y;
   for k := Header.Ymin to kmax do
      begin
      Read_PCX_Line;
      if trans=0 then ShowMCGA(x,y+k) else transput(x,k+y);
      end;
close (BlockFile);
ReadError (2);
{$I+}
end;

procedure fadein;
var i,j:integer;
    lpal:pal_array;
begin
for i:=0 to 255 do begin
     lpal[i,0]:=palette256[i,0];
     lpal[i,1]:=palette256[i,1];
     lpal[i,2]:=palette256[i,2];
     palette256[i,0]:=0;
     palette256[i,1]:=0;
     palette256[i,2]:=0;
    end;
Reg.ah := $10;                      { Set DAC Call }
Reg.al := $12;                      { set a block of DAC registers}
Reg.bx := 0;                       { first DAC register numbe     }
Reg.cx := 255;                      { number of registers to update }
Reg.dx := ofs (Palette256);          { offset of block }
Reg.es := seg (Palette256);          { segment of block }
intr ($10, Reg);

for j:=0 to 63 do  begin
for i:=0 to 255 do begin
    if palette256[i,0]<lpal[i,0] then palette256[i,0]:=palette256[i,0]+1;
    if palette256[i,1]<lpal[i,1] then palette256[i,1]:=palette256[i,1]+1;
    if palette256[i,2]<lpal[i,2] then palette256[i,2]:=palette256[i,2]+1;
    end;
Reg.ah := $10;                      { Set DAC Call }
Reg.al := $12;                      { set a block of DAC registers}
Reg.bx := 0;                       { first DAC register numbe     }
Reg.cx := 255;                      { number of registers to update }
Reg.dx := ofs (Palette256);          { offset of block }
Reg.es := seg (Palette256);          { segment of block }
intr ($10, Reg);
delay(d);
end;
end;


procedure fadeout;
var a:byte;
x:word;
    palet2 ,palet3  : array[1..$300] of byte;
begin
  reg.ax:=$1017;
  reg.bx:=0;
  reg.cx:=256;
  reg.es:=seg(palet2[1]);
  reg.dx:=ofs(palet2[1]);
  intr($10,reg);

for a:=20 downto 0 do
 begin
  for x:=4 to $300 do palet3[x]:=trunc(palet2[x]*a/20);
  reg.ax:=$1012;
  reg.bx:=0;
  reg.cx:=256;
  reg.es:=seg(palet3[1]);
  reg.dx:=ofs(palet3[1]);
  intr($10,reg);
  delay(d);
 end;

end;

procedure mcgamod;
begin
Reg.ah := $00;
Reg.al := $13;
intr ($10, Reg);
end;

procedure cls;
var
   i,j:integer;
   screen:array [0..64000] of byte absolute $A000:0000;
begin
   for i:=y to cy do
       for j:=x to cx do screen[i*320+j]:=0;
end;

procedure release;
begin
freemem(potr[numara].resim,potr[numara].uz*potr[numara].gen);
end;

procedure loadpal;
begin
assign (BlockFile, name);
reset (BlockFile, 1);                  { use 1 byte blocks }
ReadError (1);
ReadHeader;
if (header.version=5) then entirevgapalette;
end;

end.